// Edge Measure.js
//
// tg@tres-graficos.jp
// Hiroto Tsubaki
//
// Tool script for ~/Library/Application Support/Cheetah3D/scripts/Tool folder to install.
//
// update 2018/12/12: added undo code.

function buildUI( tool ) {
    
    tool.addParameterBool("only selected", 0, 0, 1, false, false);
    tool.addParameterBool("export csv", 0, 0, 1, false, false);
    tool.addParameterButton("measure", "measure", "measureEdge");
    
}

var edgeList = [];

function measureEdge( tool ) {
    var obj = tool.document().selectedObject();
    
    print( '----' );

    if (!obj || obj.family() != NGONFAMILY) return;
        
    var core = obj.core();
    edgeList.length = 0;
    
    var polygonCount = core.polygonCount();
    var polygonSize = 0;
    for(var i = 0;i < polygonCount;i++) {
        var polygonSize = core.polygonSize( i );
        for (var j = 0;j < polygonSize;j++) {
            if (tool.getParameter("only selected")) {
                if (core.edgeSelection(i, j, SELECT)) {
                    var v1 = core.vertexIndex( i, j );
                    var v2 = core.vertexIndex( i, ((j == polygonSize-1)? 0 : j + 1) );
                    edgeList.pushEdge( new Edge(v1, v2, core) );
                }
            } else {
                var v1 = core.vertexIndex( i, j );
                var v2 = core.vertexIndex( i, ((j == polygonSize-1)? 0 : j + 1) );
                edgeList.pushEdge( new Edge(v1, v2, core) );
            }
        }
    }

    for (var i = 0;i < edgeList.length;i++) {
        print(edgeList[i].toString());
    }

    if (tool.getParameter("export csv")) {
        var path = OS.runSavePanel( 'csv' );

        if (path == '') return;
        var csv = new File( path );
        csv.open(WRITE_MODE);

        if (csv.isOpen()) {
            csv.writeln( "Edge #-#,Length" );
            for (var i = 0;i < edgeList.length;i++) {
                csv.writeln( edgeList[i].toString());
            }
            csv.close();
        }
    }
}

function Edge(_v1, _v2, _core) {
    this.v1 = _v1;
    this.v2 = _v2;
    this.core = _core;
}

Array.prototype.pushEdge = function(_edge) {
    var result = -1;
    var len = this.length;
    for(var i = 0;i < len;i++) {
        if (this[i].isEqualTo(_edge)) {
            result = i;
        }
    }
    if (result == -1) this.push( _edge);
}

Edge.prototype.isEqualTo = function(_edge) {
    return ((this.v1 == _edge.v1 && this.v2 == _edge.v2) || (this.v2 == _edge.v1 && this.v1 == _edge.v2));
}

Edge.prototype.toString = function() {
    return '#' + this.v1 + ' - #' + this.v2 + "," + this.length().toFixed(3);
}

Edge.prototype.length = function() {
    var vec1 = this.core.vertex( this.v1 );
    var vec2 = this.core.vertex( this.v2 );
    var p = vec1.sub( vec2 );
    return Math.sqrt( p.x*p.x + p.y*p.y + p.z*p.z );
}
